@@ -38,6 +38,7 @@ import org.apache.flume.Clock;
 import org.apache.flume.Context;
 import org.apache.flume.Event;
 import org.apache.flume.SystemClock;
+import org.apache.flume.auth.PrivilegedExecutor;
 import org.apache.flume.instrumentation.SinkCounter;
 import org.apache.flume.sink.hdfs.HDFSEventSink.WriterCallback;
 import org.apache.hadoop.conf.Configuration;
@@ -45,7 +46,6 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.fs.Path;
 import org.apache.hadoop.io.SequenceFile.CompressionType;
 import org.apache.hadoop.io.compress.CompressionCodec;
-import org.apache.hadoop.security.UserGroupInformation;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -75,7 +75,7 @@ class BucketWriter {
   private final CompressionCodec codeC;
   private final CompressionType compType;
   private final ScheduledExecutorService timedRollerPool;
-  private final UserGroupInformation user;
+  private final PrivilegedExecutor proxyUser;
 
   private final AtomicLong fileExtensionCounter;
 
@@ -109,18 +109,18 @@ class BucketWriter {
 
   private Clock clock = new SystemClock();
   private final long retryInterval;
-  private final int maxCloseTries;
+  private final int maxRenameTries;
 
   // flag that the bucket writer was closed due to idling and thus shouldn't be
   // reopened. Not ideal, but avoids internals of owners
   protected boolean closed = false;
-  AtomicInteger closeTries = new AtomicInteger(0);
+  AtomicInteger renameTries = new AtomicInteger(0);
 
   BucketWriter(long rollInterval, long rollSize, long rollCount, long batchSize,
     Context context, String filePath, String fileName, String inUsePrefix,
     String inUseSuffix, String fileSuffix, CompressionCodec codeC,
     CompressionType compType, HDFSWriter writer,
-    ScheduledExecutorService timedRollerPool, UserGroupInformation user,
+    ScheduledExecutorService timedRollerPool, PrivilegedExecutor proxyUser,
     SinkCounter sinkCounter, int idleTimeout, WriterCallback onCloseCallback,
     String onCloseCallbackPath, long callTimeout,
     ExecutorService callTimeoutPool, long retryInterval,
@@ -138,7 +138,7 @@ class BucketWriter {
     this.compType = compType;
     this.writer = writer;
     this.timedRollerPool = timedRollerPool;
-    this.user = user;
+    this.proxyUser = proxyUser;
     this.sinkCounter = sinkCounter;
     this.idleTimeout = idleTimeout;
     this.onCloseCallback = onCloseCallback;
@@ -148,7 +148,7 @@ class BucketWriter {
     fileExtensionCounter = new AtomicLong(clock.currentTimeMillis());
 
     this.retryInterval = retryInterval;
-    this.maxCloseTries = maxCloseTries;
+    this.maxRenameTries = maxCloseTries;
     isOpen = false;
     isUnderReplicated = false;
     this.writer.configure(context);
@@ -165,33 +165,6 @@ class BucketWriter {
     this.writer = dataWriter;
   }
 
-  /**
-   * Allow methods to act as another user (typically used for HDFS Kerberos)
-   * @param <T>
-   * @param action
-   * @return
-   * @throws IOException
-   * @throws InterruptedException
-   */
-  private <T> T runPrivileged(final PrivilegedExceptionAction<T> action)
-      throws IOException, InterruptedException {
-
-    if (user != null) {
-      return user.doAs(action);
-    } else {
-      try {
-        return action.run();
-      } catch (IOException ex) {
-        throw ex;
-      } catch (InterruptedException ex) {
-        throw ex;
-      } catch (RuntimeException ex) {
-        throw ex;
-      } catch (Exception ex) {
-        throw new RuntimeException("Unexpected exception.", ex);
-      }
-    }
-  }
 
   /**
    * Clear the class counters
@@ -336,55 +309,32 @@ class BucketWriter {
       private final HDFSWriter localWriter = writer;
       @Override
       public Void call() throws Exception {
-        LOG.info("Close tries incremented");
-        closeTries.incrementAndGet();
         localWriter.close(); // could block
         return null;
       }
     };
   }
 
-  private Callable<Void> createScheduledCloseCallable(
-    final CallRunner<Void> closeCallRunner) {
+  private Callable<Void> createScheduledRenameCallable() {
 
     return new Callable<Void>() {
       private final String path = bucketPath;
       private final String finalPath = targetPath;
       private FileSystem fs = fileSystem;
-      private boolean closeSuccess = false;
-      private Path tmpFilePath = new Path(path);
-      private int closeTries = 1; // one attempt is already done
-      private final CallRunner<Void> closeCall = closeCallRunner;
+      private int renameTries = 1; // one attempt is already done
 
       @Override
       public Void call() throws Exception {
-        if (closeTries >= maxCloseTries) {
-          LOG.warn("Unsuccessfully attempted to close " + path + " " +
-            maxCloseTries + " times. File may be open, " +
-            "or may not have been renamed." );
+        if (renameTries >= maxRenameTries) {
+          LOG.warn("Unsuccessfully attempted to rename " + path + " " +
+            maxRenameTries + " times. File may still be open.");
           return null;
         }
-        closeTries++;
+        renameTries++;
         try {
-          if (!closeSuccess) {
-            if (isClosedMethod == null) {
-              LOG.debug("isFileClosed method is not available in " +
-                "the version of HDFS client being used. " +
-                "Not attempting to close file again");
-              return null;
-            }
-            if (!isFileClosed(fs, tmpFilePath)) {
-              callWithTimeout(closeCall);
-            }
-            // It is possible rename failing causes this thread
-            // to get rescheduled. In that case,
-            // don't check with NN if close succeeded as we know
-            // it did. This helps avoid an unnecessary RPC call.
-            closeSuccess = true;
-          }
           renameBucket(path, finalPath, fs);
         } catch (Exception e) {
-          LOG.warn("Closing file: " + path + " failed. Will " +
+          LOG.warn("Renaming file: " + path + " failed. Will " +
             "retry again in " + retryInterval + " seconds.", e);
           timedRollerPool.schedule(this, retryInterval,
             TimeUnit.SECONDS);
@@ -404,7 +354,11 @@ class BucketWriter {
   public synchronized void close(boolean callCloseCallback)
     throws IOException, InterruptedException {
     checkAndThrowInterruptedException();
-    flush();
+    try {
+      flush();
+    } catch (IOException e) {
+      LOG.warn("pre-close flush failed", e);
+    }
     boolean failedToClose = false;
     LOG.info("Closing {}", bucketPath);
     CallRunner<Void> closeCallRunner = createCloseCallRunner();
@@ -418,10 +372,6 @@ class BucketWriter {
             "). Exception follows.", e);
         sinkCounter.incrementConnectionFailedCount();
         failedToClose = true;
-        final Callable<Void> scheduledClose =
-          createScheduledCloseCallable(closeCallRunner);
-        timedRollerPool.schedule(scheduledClose, retryInterval,
-          TimeUnit.SECONDS);
       }
       isOpen = false;
     } else {
@@ -439,10 +389,20 @@ class BucketWriter {
       idleFuture = null;
     }
 
-    // Don't rename file if this failed to close
-    if (bucketPath != null && fileSystem != null && !failedToClose) {
+    if (bucketPath != null && fileSystem != null) {
       // could block or throw IOException
-      renameBucket(bucketPath, targetPath, fileSystem);
+      try {
+        renameBucket(bucketPath, targetPath, fileSystem);
+      } catch(Exception e) {
+        LOG.warn(
+          "failed to rename() file (" + bucketPath +
+          "). Exception follows.", e);
+        sinkCounter.incrementConnectionFailedCount();
+        final Callable<Void> scheduledRename =
+                createScheduledRenameCallable();
+        timedRollerPool.schedule(scheduledRename, retryInterval,
+                TimeUnit.SECONDS);
+      }
     }
     if (callCloseCallback) {
       runCloseAction();
@@ -667,6 +627,7 @@ class BucketWriter {
       public Void call() throws Exception {
         if (fs.exists(srcPath)) { // could block
           LOG.info("Renaming " + srcPath + " to " + dstPath);
+          renameTries.incrementAndGet();
           fs.rename(srcPath, dstPath); // could block
         }
         return null;
@@ -712,7 +673,7 @@ class BucketWriter {
     Future<T> future = callTimeoutPool.submit(new Callable<T>() {
       @Override
       public T call() throws Exception {
-        return runPrivileged(new PrivilegedExceptionAction<T>() {
+        return proxyUser.execute(new PrivilegedExceptionAction<T>() {
           @Override
           public T run() throws Exception {
             return callRunner.call();
