@@ -18,7 +18,6 @@
 
 package org.apache.flume.sink.hdfs;
 
-import java.io.File;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Calendar;
@@ -31,7 +30,6 @@ import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.ScheduledExecutorService;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicReference;
 
 import com.google.common.annotations.VisibleForTesting;
 import org.apache.flume.Channel;
@@ -41,6 +39,9 @@ import org.apache.flume.Event;
 import org.apache.flume.EventDeliveryException;
 import org.apache.flume.SystemClock;
 import org.apache.flume.Transaction;
+import org.apache.flume.auth.FlumeAuthenticationUtil;
+import org.apache.flume.auth.FlumeAuthenticator;
+import org.apache.flume.auth.PrivilegedExecutor;
 import org.apache.flume.conf.Configurable;
 import org.apache.flume.formatter.output.BucketPath;
 import org.apache.flume.instrumentation.SinkCounter;
@@ -50,9 +51,6 @@ import org.apache.hadoop.fs.FileSystem;
 import org.apache.hadoop.io.SequenceFile.CompressionType;
 import org.apache.hadoop.io.compress.CompressionCodec;
 import org.apache.hadoop.io.compress.CompressionCodecFactory;
-import org.apache.hadoop.security.SecurityUtil;
-import org.apache.hadoop.security.UserGroupInformation;
-import org.apache.hadoop.security.UserGroupInformation.AuthenticationMethod;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -100,12 +98,6 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
   private static final int defaultThreadPoolSize = 10;
   private static final int defaultRollTimerPoolSize = 1;
 
-  /**
-   * Singleton credential manager that manages static credentials for the
-   * entire JVM
-   */
-  private static final AtomicReference<KerberosUser> staticLogin
-      = new AtomicReference<KerberosUser>();
 
   private final HDFSWriterFactory writerFactory;
   private WriterLinkedHashMap sfWriters;
@@ -129,11 +121,6 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
   private ExecutorService callTimeoutPool;
   private ScheduledExecutorService timedRollerPool;
 
-  private String kerbConfPrincipal;
-  private String kerbKeytab;
-  private String proxyUserName;
-  private UserGroupInformation proxyTicket;
-
   private boolean needRounding = false;
   private int roundUnit = Calendar.SECOND;
   private int roundValue = 1;
@@ -150,6 +137,7 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
   private final Object sfWritersLock = new Object();
   private long retryInterval;
   private int tryCount;
+  private PrivilegedExecutor privExecutor;
 
 
   /*
@@ -225,9 +213,9 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
         defaultThreadPoolSize);
     rollTimerPoolSize = context.getInteger("hdfs.rollTimerPoolSize",
         defaultRollTimerPoolSize);
-    kerbConfPrincipal = context.getString("hdfs.kerberosPrincipal", "");
-    kerbKeytab = context.getString("hdfs.kerberosKeytab", "");
-    proxyUserName = context.getString("hdfs.proxyUser", "");
+    String kerbConfPrincipal = context.getString("hdfs.kerberosPrincipal");
+    String kerbKeytab = context.getString("hdfs.kerberosKeytab");
+    String proxyUser = context.getString("hdfs.proxyUser");
     tryCount = context.getInteger("hdfs.closeTries", defaultTryCount);
     if(tryCount <= 0) {
       LOG.warn("Retry count value : " + tryCount + " is not " +
@@ -269,9 +257,13 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
           + " when fileType is: " + fileType);
     }
 
-    if (!authenticate()) {
-      LOG.error("Failed to authenticate!");
-    }
+    // get the appropriate executor
+    this.privExecutor = FlumeAuthenticationUtil.getAuthenticator(
+            kerbConfPrincipal, kerbKeytab).proxyAs(proxyUser);
+
+
+
+
     needRounding = context.getBoolean("hdfs.round", false);
 
     if(needRounding) {
@@ -482,7 +474,7 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
       rollSize, rollCount,
       batchSize, context, realPath, realName, inUsePrefix, inUseSuffix,
       suffix, codeC, compType, hdfsWriter, timedRollerPool,
-      proxyTicket, sinkCounter, idleTimeout, closeCallback,
+      privExecutor, sinkCounter, idleTimeout, closeCallback,
       lookupPath, callTimeout, callTimeoutPool, retryInterval,
       tryCount);
     if(mockFs != null) {
@@ -495,16 +487,18 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
   @Override
   public void stop() {
     // do not constrain close() calls with a timeout
-    for (Entry<String, BucketWriter> entry : sfWriters.entrySet()) {
-      LOG.info("Closing {}", entry.getKey());
+    synchronized (sfWritersLock) {
+      for (Entry<String, BucketWriter> entry : sfWriters.entrySet()) {
+        LOG.info("Closing {}", entry.getKey());
 
-      try {
-        entry.getValue().close();
-      } catch (Exception ex) {
-        LOG.warn("Exception while closing " + entry.getKey() + ". " +
-                "Exception follows.", ex);
-        if (ex instanceof InterruptedException) {
-          Thread.currentThread().interrupt();
+        try {
+          entry.getValue().close();
+        } catch (Exception ex) {
+          LOG.warn("Exception while closing " + entry.getKey() + ". " +
+                  "Exception follows.", ex);
+          if (ex instanceof InterruptedException) {
+            Thread.currentThread().interrupt();
+          }
         }
       }
     }
@@ -526,8 +520,10 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
     callTimeoutPool = null;
     timedRollerPool = null;
 
-    sfWriters.clear();
-    sfWriters = null;
+    synchronized (sfWritersLock) {
+      sfWriters.clear();
+      sfWriters = null;
+    }
     sinkCounter.stop();
     super.stop();
   }
@@ -547,197 +543,6 @@ public class HDFSEventSink extends AbstractSink implements Configurable {
     super.start();
   }
 
-  private boolean authenticate() {
-
-    // logic for kerberos login
-    boolean useSecurity = UserGroupInformation.isSecurityEnabled();
-
-    LOG.info("Hadoop Security enabled: " + useSecurity);
-
-    if (useSecurity) {
-
-      // sanity checking
-      if (kerbConfPrincipal.isEmpty()) {
-        LOG.error("Hadoop running in secure mode, but Flume config doesn't "
-                + "specify a principal to use for Kerberos auth.");
-        return false;
-      }
-      if (kerbKeytab.isEmpty()) {
-        LOG.error("Hadoop running in secure mode, but Flume config doesn't "
-                + "specify a keytab to use for Kerberos auth.");
-        return false;
-      } else {
-        //If keytab is specified, user should want it take effect.
-        //HDFSEventSink will halt when keytab file is non-exist or unreadable
-        File kfile = new File(kerbKeytab);
-        if (!(kfile.isFile() && kfile.canRead())) {
-          throw new IllegalArgumentException("The keyTab file: "
-                  + kerbKeytab + " is nonexistent or can't read. "
-                  + "Please specify a readable keytab file for Kerberos auth.");
-        }
-      }
-
-      String principal;
-      try {
-        // resolves _HOST pattern using standard Hadoop search/replace
-        // via DNS lookup when 2nd argument is empty
-        principal = SecurityUtil.getServerPrincipal(kerbConfPrincipal, "");
-      } catch (IOException e) {
-        LOG.error("Host lookup error resolving kerberos principal ("
-                + kerbConfPrincipal + "). Exception follows.", e);
-        return false;
-      }
-
-      Preconditions.checkNotNull(principal, "Principal must not be null");
-      KerberosUser prevUser = staticLogin.get();
-      KerberosUser newUser = new KerberosUser(principal, kerbKeytab);
-
-      // be cruel and unusual when user tries to login as multiple principals
-      // this isn't really valid with a reconfigure but this should be rare
-      // enough to warrant a restart of the agent JVM
-      // TODO: find a way to interrogate the entire current config state,
-      // since we don't have to be unnecessarily protective if they switch all
-      // HDFS sinks to use a different principal all at once.
-      Preconditions.checkState(prevUser == null || prevUser.equals(newUser),
-              "Cannot use multiple kerberos principals in the same agent. " +
-                      " Must restart agent to use new principal or keytab. " +
-                      "Previous = %s, New = %s", prevUser, newUser);
-
-      // attempt to use cached credential if the user is the same
-      // this is polite and should avoid flooding the KDC with auth requests
-      UserGroupInformation curUser = null;
-      if (prevUser != null && prevUser.equals(newUser)) {
-        try {
-          curUser = UserGroupInformation.getLoginUser();
-        } catch (IOException e) {
-          LOG.warn("User unexpectedly had no active login. Continuing with " +
-                  "authentication", e);
-        }
-      }
-
-      if (curUser == null || !curUser.getUserName().equals(principal)) {
-        try {
-          // static login
-          kerberosLogin(this, principal, kerbKeytab);
-        } catch (IOException e) {
-          LOG.error("Authentication or file read error while attempting to "
-                  + "login as kerberos principal (" + principal + ") using "
-                  + "keytab (" + kerbKeytab + "). Exception follows.", e);
-          return false;
-        }
-      } else {
-        LOG.debug("{}: Using existing principal login: {}", this, curUser);
-      }
-
-      // we supposedly got through this unscathed... so store the static user
-      staticLogin.set(newUser);
-    }
-
-    // hadoop impersonation works with or without kerberos security
-    proxyTicket = null;
-    if (!proxyUserName.isEmpty()) {
-      try {
-        proxyTicket = UserGroupInformation.createProxyUser(
-                proxyUserName, UserGroupInformation.getLoginUser());
-      } catch (IOException e) {
-        LOG.error("Unable to login as proxy user. Exception follows.", e);
-        return false;
-      }
-    }
-
-    UserGroupInformation ugi = null;
-    if (proxyTicket != null) {
-      ugi = proxyTicket;
-    } else if (useSecurity) {
-      try {
-        ugi = UserGroupInformation.getLoginUser();
-      } catch (IOException e) {
-        LOG.error("Unexpected error: Unable to get authenticated user after " +
-                "apparent successful login! Exception follows.", e);
-        return false;
-      }
-    }
-
-    if (ugi != null) {
-      // dump login information
-      AuthenticationMethod authMethod = ugi.getAuthenticationMethod();
-      LOG.info("Auth method: {}", authMethod);
-      LOG.info(" User name: {}", ugi.getUserName());
-      LOG.info(" Using keytab: {}", ugi.isFromKeytab());
-      if (authMethod == AuthenticationMethod.PROXY) {
-        UserGroupInformation superUser;
-        try {
-          superUser = UserGroupInformation.getLoginUser();
-          LOG.info(" Superuser auth: {}", superUser.getAuthenticationMethod());
-          LOG.info(" Superuser name: {}", superUser.getUserName());
-          LOG.info(" Superuser using keytab: {}", superUser.isFromKeytab());
-        } catch (IOException e) {
-          LOG.error("Unexpected error: unknown superuser impersonating proxy.",
-                  e);
-          return false;
-        }
-      }
-
-      LOG.info("Logged in as user {}", ugi.getUserName());
-
-      return true;
-    }
-
-    return true;
-  }
-
-  /**
-   * Static synchronized method for static Kerberos login. <br/>
-   * Static synchronized due to a thundering herd problem when multiple Sinks
-   * attempt to log in using the same principal at the same time with the
-   * intention of impersonating different users (or even the same user).
-   * If this is not controlled, MIT Kerberos v5 believes it is seeing a replay
-   * attach and it returns:
-   * <blockquote>Request is a replay (34) - PROCESS_TGS</blockquote>
-   * In addition, since the underlying Hadoop APIs we are using for
-   * impersonation are static, we define this method as static as well.
-   *
-   * @param principal
-   *         Fully-qualified principal to use for authentication.
-   * @param keytab
-   *         Location of keytab file containing credentials for principal.
-   * @return Logged-in user
-   * @throws IOException
-   *         if login fails.
-   */
-  private static synchronized UserGroupInformation kerberosLogin(
-          HDFSEventSink sink, String principal, String keytab) throws IOException {
-
-    // if we are the 2nd user thru the lock, the login should already be
-    // available statically if login was successful
-    UserGroupInformation curUser = null;
-    try {
-      curUser = UserGroupInformation.getLoginUser();
-    } catch (IOException e) {
-      // not a big deal but this shouldn't typically happen because it will
-      // generally fall back to the UNIX user
-      LOG.debug("Unable to get login user before Kerberos auth attempt.", e);
-    }
-
-    // we already have logged in successfully
-    if (curUser != null && curUser.getUserName().equals(principal)) {
-      LOG.debug("{}: Using existing principal ({}): {}",
-              new Object[]{sink, principal, curUser});
-
-      // no principal found
-    } else {
-
-      LOG.info("{}: Attempting kerberos login as principal ({}) from keytab " +
-              "file ({})", new Object[]{sink, principal, keytab});
-
-      // attempt static kerberos login
-      UserGroupInformation.loginUserFromKeytab(principal, keytab);
-      curUser = UserGroupInformation.getLoginUser();
-    }
-
-    return curUser;
-  }
-
   @Override
   public String toString() {
     return "{ Sink type:" + getClass().getSimpleName() + ", name:" + getName() +
