@@ -110,7 +110,7 @@ public class HDFSSequenceFile extends AbstractHDFSWriter {
 
   @Override
   public void sync() throws IOException {
-    writer.syncFs();
+    hflushOrSync(outStream);
   }
 
   @Override
