@@ -29,6 +29,8 @@ import java.util.concurrent.atomic.AtomicBoolean;
 import org.apache.flume.Clock;
 import org.apache.flume.Context;
 import org.apache.flume.Event;
+import org.apache.flume.auth.FlumeAuthenticationUtil;
+import org.apache.flume.auth.PrivilegedExecutor;
 import org.apache.flume.event.EventBuilder;
 import org.apache.flume.instrumentation.SinkCounter;
 import org.apache.hadoop.conf.Configuration;
@@ -53,10 +55,12 @@ public class TestBucketWriter {
   private Context ctx = new Context();
 
   private static ScheduledExecutorService timedRollerPool;
+  private static PrivilegedExecutor proxy;
 
   @BeforeClass
   public static void setup() {
     timedRollerPool = Executors.newSingleThreadScheduledExecutor();
+    proxy = FlumeAuthenticationUtil.getAuthenticator(null, null).proxyAs(null);
   }
 
   @AfterClass
@@ -72,7 +76,7 @@ public class TestBucketWriter {
     MockHDFSWriter hdfsWriter = new MockHDFSWriter();
     BucketWriter bucketWriter = new BucketWriter(0, 0, maxEvents, 0, ctx,
         "/tmp", "file", "", ".tmp", null, null, SequenceFile.CompressionType.NONE,
-        hdfsWriter, timedRollerPool, null,
+        hdfsWriter, timedRollerPool, proxy,
         new SinkCounter("test-bucket-writer-" + System.currentTimeMillis()), 0,
         null, null, 30000, Executors.newSingleThreadExecutor(), 0, 0);
 
@@ -97,7 +101,7 @@ public class TestBucketWriter {
     BucketWriter bucketWriter = new BucketWriter(0, maxBytes, 0, 0,
       ctx, "/tmp", "file", "", ".tmp", null, null,
       SequenceFile.CompressionType.NONE, hdfsWriter,timedRollerPool,
-      null, new SinkCounter("test-bucket-writer-" +
+      proxy, new SinkCounter("test-bucket-writer-" +
       System.currentTimeMillis()),0, null, null, 30000,
       Executors.newSingleThreadExecutor(), 0, 0);
 
@@ -124,7 +128,7 @@ public class TestBucketWriter {
     MockHDFSWriter hdfsWriter = new MockHDFSWriter();
     BucketWriter bucketWriter = new BucketWriter(ROLL_INTERVAL, 0, 0, 0, ctx,
       "/tmp", "file", "", ".tmp", null, null, SequenceFile.CompressionType.NONE,
-      hdfsWriter, timedRollerPool, null,
+      hdfsWriter, timedRollerPool, proxy,
       new SinkCounter("test-bucket-writer-" + System.currentTimeMillis()),
       0, new HDFSEventSink.WriterCallback() {
       @Override
@@ -147,7 +151,7 @@ public class TestBucketWriter {
 
     bucketWriter = new BucketWriter(ROLL_INTERVAL, 0, 0, 0, ctx,
       "/tmp", "file", "", ".tmp", null, null, SequenceFile.CompressionType.NONE,
-      hdfsWriter, timedRollerPool, null,
+      hdfsWriter, timedRollerPool, proxy,
       new SinkCounter("test-bucket-writer-"
         + System.currentTimeMillis()), 0, null, null, 30000,
       Executors.newSingleThreadExecutor(), 0, 0);
@@ -230,7 +234,7 @@ public class TestBucketWriter {
     BucketWriter bucketWriter = new BucketWriter(ROLL_INTERVAL, 0, 0,
       0, ctx, path, name, "", ".tmp", null, null,
       SequenceFile.CompressionType.NONE, hdfsWriter,
-      timedRollerPool, null, new SinkCounter("test-bucket-writer-"
+      timedRollerPool, proxy, new SinkCounter("test-bucket-writer-"
       + System.currentTimeMillis()),
       0, null, null, 30000, Executors.newSingleThreadExecutor(),
       0, 0);
@@ -255,7 +259,7 @@ public class TestBucketWriter {
       BucketWriter bucketWriter = new BucketWriter(ROLL_INTERVAL, 0,
         0, 0, ctx, "/tmp", "file", "", ".tmp", suffix, null,
         SequenceFile.CompressionType.NONE, hdfsWriter,
-        timedRollerPool, null, new SinkCounter("test-bucket-writer-"
+        timedRollerPool, proxy, new SinkCounter("test-bucket-writer-"
         + System.currentTimeMillis()), 0, null, null, 30000,
         Executors.newSingleThreadExecutor(), 0, 0);
 
@@ -283,7 +287,7 @@ public class TestBucketWriter {
       BucketWriter bucketWriter = new BucketWriter(ROLL_INTERVAL, 0,
         0, 0, ctx, "/tmp", "file", "", ".tmp", suffix, null,
         SequenceFile.CompressionType.NONE, hdfsWriter,
-        timedRollerPool, null, new SinkCounter(
+        timedRollerPool, proxy, new SinkCounter(
         "test-bucket-writer-" + System.currentTimeMillis()), 0,
         null, null, 30000, Executors.newSingleThreadExecutor(), 0, 0);
 
@@ -316,7 +320,7 @@ public class TestBucketWriter {
       0, ctx, "/tmp", "file", "", ".tmp", suffix,
       HDFSEventSink.getCodec("gzip"),
       SequenceFile.CompressionType.BLOCK, hdfsWriter,
-      timedRollerPool, null, new SinkCounter("test-bucket-writer-"
+      timedRollerPool, proxy, new SinkCounter("test-bucket-writer-"
       + System.currentTimeMillis()), 0, null, null, 30000,
       Executors.newSingleThreadExecutor(), 0, 0
     );
@@ -348,7 +352,7 @@ public class TestBucketWriter {
     BucketWriter bucketWriter = new BucketWriter(ROLL_INTERVAL, 0, 0,
       0, ctx, "/tmp", "file", PREFIX, ".tmp", null, null,
       SequenceFile.CompressionType.NONE, hdfsWriter,
-      timedRollerPool, null, new SinkCounter(
+      timedRollerPool, proxy, new SinkCounter(
         "test-bucket-writer-" + System.currentTimeMillis()), 0,
       null, null, 30000, Executors.newSingleThreadExecutor(), 0, 0);
 
@@ -368,7 +372,7 @@ public class TestBucketWriter {
     BucketWriter bucketWriter = new BucketWriter(ROLL_INTERVAL, 0, 0,
       0, ctx, "/tmp", "file", "", SUFFIX, null, null,
       SequenceFile.CompressionType.NONE, hdfsWriter,
-      timedRollerPool, null, new SinkCounter(
+      timedRollerPool, proxy, new SinkCounter(
         "test-bucket-writer-" + System.currentTimeMillis()), 0,
       null, null, 30000, Executors.newSingleThreadExecutor(), 0, 0);
 
@@ -388,7 +392,7 @@ public class TestBucketWriter {
     BucketWriter bucketWriter = new BucketWriter(ROLL_INTERVAL, 0, 0,
       0, ctx, "/tmp", "file", "", SUFFIX, null, null,
       SequenceFile.CompressionType.NONE,
-      hdfsWriter, timedRollerPool, null,
+      hdfsWriter, timedRollerPool, proxy,
       new SinkCounter(
         "test-bucket-writer-" + System.currentTimeMillis()), 0,
       new HDFSEventSink.WriterCallback() {
@@ -408,15 +412,18 @@ public class TestBucketWriter {
 
 
   @Test
-  public void testSequenceFileCloseRetries() throws Exception {
-    SequenceFileCloseRetryCoreTest(1);
-    SequenceFileCloseRetryCoreTest(5);
-    SequenceFileCloseRetryCoreTest(2);
+  public void testSequenceFileRenameRetries() throws Exception {
+    SequenceFileRenameRetryCoreTest(1, true);
+    SequenceFileRenameRetryCoreTest(5, true);
+    SequenceFileRenameRetryCoreTest(2, true);
+
+    SequenceFileRenameRetryCoreTest(1, false);
+    SequenceFileRenameRetryCoreTest(5, false);
+    SequenceFileRenameRetryCoreTest(2, false);
 
   }
 
-
-  public void SequenceFileCloseRetryCoreTest(int numberOfClosesRequired) throws Exception {
+  public void SequenceFileRenameRetryCoreTest(int numberOfRetriesRequired, boolean closeSucceed) throws Exception {
     String hdfsPath = "file:///tmp/flume-test."
       + Calendar.getInstance().getTimeInMillis() + "."
       + Thread.currentThread().getId();
@@ -429,21 +436,21 @@ public class TestBucketWriter {
     fs.mkdirs(dirPath);
     context.put("hdfs.path", hdfsPath);
     context.put("hdfs.closeTries",
-      String.valueOf(numberOfClosesRequired));
+      String.valueOf(numberOfRetriesRequired));
     context.put("hdfs.rollCount", "1");
     context.put("hdfs.retryInterval", "1");
     context.put("hdfs.callTimeout", Long.toString(1000));
     MockFileSystem mockFs = new
       MockFileSystem(fs,
-      numberOfClosesRequired);
+      numberOfRetriesRequired, closeSucceed);
     BucketWriter bucketWriter = new BucketWriter(0, 0, 1, 1, ctx,
       hdfsPath, hdfsPath, "singleBucket", ".tmp", null, null,
       null, new MockDataStream(mockFs),
-      timedRollerPool, null,
+      timedRollerPool, proxy,
       new SinkCounter(
         "test-bucket-writer-" + System.currentTimeMillis()),
       0, null, null, 30000, Executors.newSingleThreadExecutor(), 1,
-      numberOfClosesRequired);
+      numberOfRetriesRequired);
 
     bucketWriter.setFileSystem(mockFs);
     // At this point, we checked if isFileClosed is available in
@@ -453,12 +460,11 @@ public class TestBucketWriter {
     // This is what triggers the close, so a 2nd append is required :/
     bucketWriter.append(event);
 
-    TimeUnit.SECONDS.sleep(numberOfClosesRequired + 2);
+    TimeUnit.SECONDS.sleep(numberOfRetriesRequired + 2);
 
-    int expectedNumberOfCloses = numberOfClosesRequired;
-    Assert.assertTrue("Expected " + expectedNumberOfCloses + " " +
-      "but got " + bucketWriter.closeTries.get(),
-      bucketWriter.closeTries.get() ==
-        expectedNumberOfCloses);
+    Assert.assertTrue("Expected " + numberOfRetriesRequired + " " +
+      "but got " + bucketWriter.renameTries.get(),
+      bucketWriter.renameTries.get() ==
+        numberOfRetriesRequired);
   }
 }
