@@ -20,6 +20,7 @@ package org.apache.flume.source.jms;
 import java.io.File;
 import java.io.IOException;
 import java.util.List;
+import java.util.Locale;
 import java.util.Properties;
 
 import javax.jms.ConnectionFactory;
@@ -64,6 +65,7 @@ public class JMSSource extends AbstractPollableSource {
   private String providerUrl;
   private String destinationName;
   private JMSDestinationType destinationType;
+  private JMSDestinationLocator destinationLocator;
   private String messageSelector;
   private Optional<String> userName;
   private Optional<String> password;
@@ -72,6 +74,7 @@ public class JMSSource extends AbstractPollableSource {
   private long pollTimeout;
 
   private int jmsExceptionCounter;
+  private InitialContext initialContext;
 
 
   public JMSSource() {
@@ -99,7 +102,11 @@ public class JMSSource extends AbstractPollableSource {
         DESTINATION_NAME, "").trim();
 
     String destinationTypeName = context.getString(JMSSourceConfiguration.
-        DESTINATION_TYPE, "").trim().toUpperCase();
+        DESTINATION_TYPE, "").trim().toUpperCase(Locale.ENGLISH);
+
+    String destinationLocatorName = context.getString(JMSSourceConfiguration.
+        DESTINATION_LOCATOR, JMSSourceConfiguration.DESTINATION_LOCATOR_DEFAULT)
+      .trim().toUpperCase(Locale.ENGLISH);
 
     messageSelector = context.getString(JMSSourceConfiguration.
         MESSAGE_SELECTOR, "").trim();
@@ -196,11 +203,16 @@ public class JMSSource extends AbstractPollableSource {
           "invalid.", destinationTypeName), e);
     }
 
+    try {
+      destinationLocator = JMSDestinationLocator.valueOf(destinationLocatorName);
+    } catch (IllegalArgumentException e) {
+      throw new FlumeException(String.format("Destination locator '%s' is " +
+          "invalid.", destinationLocatorName), e);
+    }
+
     Preconditions.checkArgument(batchSize > 0, "Batch size must be greater " +
         "than 0");
 
-    InitialContext initalContext;
-
     try {
       Properties contextProperties = new Properties();
       contextProperties.setProperty(
@@ -208,7 +220,7 @@ public class JMSSource extends AbstractPollableSource {
           initialContextFactoryName);
       contextProperties.setProperty(
           javax.naming.Context.PROVIDER_URL, providerUrl);
-      initalContext = initialContextFactory.create(contextProperties);
+      initialContext = initialContextFactory.create(contextProperties);
     } catch (NamingException e) {
       throw new FlumeException(String.format(
           "Could not create initial context %s provider %s",
@@ -216,7 +228,7 @@ public class JMSSource extends AbstractPollableSource {
     }
 
     try {
-      connectionFactory = (ConnectionFactory) initalContext.
+      connectionFactory = (ConnectionFactory) initialContext.
           lookup(connectionFactoryName);
     } catch (NamingException e) {
       throw new FlumeException("Could not lookup ConnectionFactory", e);
@@ -302,8 +314,9 @@ public class JMSSource extends AbstractPollableSource {
   }
   private JMSMessageConsumer createConsumer() throws JMSException {
     logger.info("Creating new consumer for " + destinationName);
-    JMSMessageConsumer consumer = consumerFactory.create(connectionFactory,
-        destinationName, destinationType, messageSelector, batchSize,
+    JMSMessageConsumer consumer = consumerFactory.create(initialContext,
+    connectionFactory, destinationName, destinationType, destinationLocator,
+    messageSelector, batchSize,
         pollTimeout, converter, userName, password);
     jmsExceptionCounter = 0;
     return consumer;
